/*
 * Copyright (C) 2007-2013 Crafter Software Corporation.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.craftercms.core.util.generators.impl;

import org.craftercms.core.util.generators.TimestampGenerator;
import org.craftercms.core.util.generators.TimestampGenerator;

import java.util.concurrent.atomic.AtomicLong;

/**
 * Implementation of {@link org.craftercms.core.util.generators.TimestampGenerator}
 * that returns increments of a counter as timestamps.
 *
 * @author Alfonso Vásquez
 */
public class IncrementalTimestampGenerator implements TimestampGenerator {

    private AtomicLong counter;

    public IncrementalTimestampGenerator() {
        counter = new AtomicLong(0);
    }

    /**
     * Returns increments of a counter as timestamps.
     *
     * @return
     *      the timestamp
     */
    public long generate() {
        return counter.getAndIncrement();
    }
}
